@@ -4,7 +4,7 @@ use Digest::MD5 'md5';
 use Carp;
 use strict;
 use vars qw($VERSION);
-$VERSION = '2.19';
+$VERSION = '2.21';
 
 use constant RANDOM_DEVICE => '/dev/urandom';
 
@@ -19,7 +19,7 @@ sub new {
     }
 
     # CGI style arguments
-    elsif ($_[0] =~ /^-[a-zA-Z]{1,20}$/) {
+    elsif ($_[0] =~ /^-[a-zA-Z_]{1,20}$/) {
       my %tmp = @_;
       while ( my($key,$value) = each %tmp) {
 	$key =~ s/^-//;
@@ -32,10 +32,20 @@ sub new {
 	$options->{cipher} = shift;
     }
 
+    my $cipher_object_provided = $options->{cipher} && ref $options->{cipher};
+
     # "key" is a misnomer here, because it is actually usually a passphrase that is used
     # to derive the true key
     my $pass = $options->{key};
-    croak "Please provide an encryption/decryption passphrase or key using -key" unless defined $pass;
+
+    if ($cipher_object_provided) {
+      carp "Both a key and a pre-initialized Crypt::* object were passed. The key will be ignored"
+	if defined $pass;
+      $pass ||= '';
+    }
+    elsif (!defined $pass) {
+      croak "Please provide an encryption/decryption passphrase or key using -key"
+    }
 
     # header mode
     my %valid_modes = map {$_=>1} qw(none salt randomiv);
@@ -50,11 +60,14 @@ sub new {
 
     my $cipher = $options->{cipher};
     $cipher = 'Crypt::DES' unless $cipher;
-    $cipher = $cipher=~/^Crypt::/ ? $cipher : "Crypt::$cipher";
-    $cipher->can('encrypt') or eval "require $cipher; 1" or croak "Couldn't load $cipher: $@";
+    my $cipherclass = ref $cipher || $cipher;
 
-    # some crypt modules use the class Crypt::, and others don't
-    $cipher =~ s/^Crypt::// unless $cipher->can('keysize');
+    unless (ref $cipher) {  # munge the class name if no object passed
+      $cipher = $cipher=~/^Crypt::/ ? $cipher : "Crypt::$cipher";
+      $cipher->can('encrypt') or eval "require $cipher; 1" or croak "Couldn't load $cipher: $@";
+      # some crypt modules use the class Crypt::, and others don't
+      $cipher =~ s/^Crypt::// unless $cipher->can('keysize');
+    }
 
     # allow user to override these values
     my $ks        = $options->{keysize};
@@ -68,7 +81,7 @@ sub new {
     # keysize (well, Crypt::Blowfish in any case).  If we detect
     # this, and find the blowfish module in use, then assume 56.
     # Otherwise assume the least common denominator of 8.
-    $ks ||= $cipher =~ /blowfish/i ? 56 : 8;
+    $ks ||= $cipherclass =~ /blowfish/i ? 56 : 8;
     $bs ||= $ks;
 
     my $pcbc = $options->{'pcbc'};
@@ -88,7 +101,7 @@ sub new {
     # note: iv will be autogenerated by start() if not specified in options
     my $iv = $options->{iv};
     my $random_iv = 1 unless defined $iv;
-    croak "Initialization vector must be exactly $bs bytes long when using the $cipher cipher" if defined $iv and length($iv) != $bs;
+    croak "Initialization vector must be exactly $bs bytes long when using the $cipherclass cipher" if defined $iv and length($iv) != $bs;
 
     my $legacy_hack = $options->{insecure_legacy_decrypt};
     my $padding     = $options->{padding} || 'standard';
@@ -306,9 +319,9 @@ sub _generate_iv_and_cipher_from_datastream {
     unless $self->{key} && $self->{civ};
 
   # now we can generate the crypt object itself
-  $self->{crypt} = $self->{cipher}->new($self->{key})
-    or croak "Could not create $self->{cipher} object: $@";
-
+  $self->{crypt} = ref $self->{cipher} ? $self->{cipher}
+                                       : $self->{cipher}->new($self->{key})
+					 or croak "Could not create $self->{cipher} object: $@";
   return '';
 }
 
@@ -348,9 +361,9 @@ sub _generate_iv_and_cipher_from_options {
 
   croak "key and/or iv are missing" unless defined $self->{key} && defined $self->{civ};
 
-  $self->{crypt} = $self->{cipher}->new($self->{key})
-    or croak "Could not create $self->{cipher} object: $@";
-
+  $self->{crypt} = ref $self->{cipher} ? $self->{cipher}
+                                       : $self->{cipher}->new($self->{key})
+					 or croak "Could not create $self->{cipher} object: $@";
   return $result;
 }
 
@@ -587,7 +600,8 @@ The new() method creates a new Crypt::CBC object. It accepts a list of
 
   -key            The encryption/decryption key (required)
 
-  -cipher         The cipher algorithm (defaults to Crypt::DES)
+  -cipher         The cipher algorithm (defaults to Crypt::DES), or
+                     a preexisting cipher object.
 
   -salt           Enables OpenSSL-compatibility. If equal to a value
                     of "1" then causes a random salt to be generated
@@ -659,6 +673,14 @@ Crypt::DES, Crypt::DES_EDE3, Crypt::IDEA, Crypt::Blowfish,
 Crypt::CAST5 and Crypt::Rijndael. You may refer to them using their
 full names ("Crypt::IDEA") or in abbreviated form ("IDEA").
 
+Instead of passing the name of a cipher class, you may pass an
+already-created block cipher object. This allows you to take advantage
+of cipher algorithms that have parameterized new() methods, such as
+Crypt::Eksblowfish:
+
+  my $eksblowfish = Crypt::Eksblowfish->new(8,$salt,$key);
+  my $cbc         = Crypt::CBC->new(-cipher=>$eksblowfish);
+
 The B<-key> argument provides either a passphrase to use to generate
 the encryption key, or the literal value of the block cipher key. If
 used in passphrase mode (which is the default), B<-key> can be any
@@ -670,6 +692,9 @@ operation and specify the key directly, pass a true value to the
 B<-literal_key> option. In this case, you should choose a key of
 length exactly equal to the cipher's key length.
 
+If you pass an existing Crypt::* object to new(), then the -key
+argument is ignored and the module will generate a warning.
+
 The B<-header> argument specifies what type of header, if any, to
 prepend to the beginning of the encrypted data stream. The header
 allows Crypt::CBC to regenerate the original IV and correctly decrypt
